# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# ***** END GPL LICENSE BLOCK *****
import bpy

def draw_add_none_destructive_primitive(self, context):
    if context.mode == 'OBJECT':
        layout = self.layout    
        layout.label(text="None-Destructive", icon='FILE_3D')
        layout.operator("mesh.add_none_destructive_primitive", text="Grid", icon='MESH_PLANE').type = 'Grid'
        layout.operator("mesh.add_none_destructive_primitive", text="Cube", icon='MESH_CUBE').type = 'Cube'
        layout.operator("mesh.add_none_destructive_primitive", text="Circle", icon='MESH_CIRCLE').type = 'Circle'
        layout.operator("mesh.add_none_destructive_primitive", text="UV Sphere", icon='MESH_UVSPHERE').type = 'UV Sphere'
        layout.operator("mesh.add_none_destructive_primitive", text="Ico Sphere", icon='MESH_ICOSPHERE').type = 'Ico Sphere'
        layout.operator("mesh.add_none_destructive_primitive", text="Cylinder", icon='MESH_CYLINDER').type = 'Cylinder'
        layout.operator("mesh.add_none_destructive_primitive", text="Cone", icon='MESH_CONE').type = 'Cone'
        layout.operator("mesh.add_none_destructive_primitive", text="Torus", icon='MESH_TORUS').type = 'Torus'
        layout.operator("mesh.add_none_destructive_primitive", text="Quad Sphere", icon='SPHERE').type = 'Quad Sphere'
        layout.operator("mesh.add_none_destructive_primitive", text="Spiral", icon='MOD_SCREW').type = 'Spiral'
        layout.separator()

class AddNoneDestructivePrimitive(bpy.types.Operator):
    bl_idname = "mesh.add_none_destructive_primitive"
    bl_label = "Add None Destructive Primitive"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Adds a none destructive primitive"

    type: bpy.props.StringProperty()

    def draw(self, context):
        text = "Added: " + self.type
        self.layout.label(text=text, icon='FILE_3D')


    def execute(self, context):
        bpy.ops.object.select_all(action='DESELECT')

        file_path = __file__.replace('__init__.py', 'Primitives.blend')
        name = self.type

        with bpy.data.libraries.load(file_path, link=False) as (data_from, data_to):
            data_to.node_groups = [name for name in data_from.node_groups if name == name]

        # add empty mesh object with none bpy ops
        new_mesh = bpy.data.meshes.new(name)
        new_obj = bpy.data.objects.new(name, new_mesh)
        context.collection.objects.link(new_obj)
        # move to cursor
        new_obj.location = context.scene.cursor.location

        # add geometry node modifier to object
        new_obj.modifiers.new(name=name, type='NODES')
        new_obj.modifiers[name].node_group = bpy.data.node_groups[name]

        # set active object + select
        new_obj.select_set(True)
        context.view_layer.objects.active = new_obj
        
        # add object properties
        new_obj['none_destructive_primitive'] = name

        # register timer to switch to modifier context
        bpy.app.timers.register(switch_to_modifier_context, first_interval=0.001) # workaround to switch to modifier context error if it does not exist already
        return {'FINISHED'}
    
def switch_to_modifier_context():
    for area in bpy.context.screen.areas:
        if area.type == 'PROPERTIES':
            area.spaces[0].context = 'MODIFIER'

class ApplyPrimitiveGeometryNodesModifier(bpy.types.Operator):
    bl_idname = "object.apply_geometry_nodes_modifier"
    bl_label = "Apply Geometry Nodes Modifier"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Apply the geometry nodes modifier to the selected object"

    def execute(self, context):
        for obj in context.selected_objects:
            if obj and obj.type == 'MESH' and obj.modifiers:
                if 'none_destructive_primitive' in obj:
                    mod_name = obj['none_destructive_primitive']
                    if mod_name:
                        for mod in obj.modifiers:
                            if mod.name == mod_name:
                                if mod.show_viewport:
                                    context.view_layer.objects.active = obj
                                    mod.show_viewport = False
                                    bpy.ops.object.modifier_copy(modifier=mod_name)
                                    bpy.ops.object.modifier_apply(modifier=mod_name)

                                    active_modifier = obj.modifiers.active
                                    active_modifier.name = mod_name
                  
        return {'PASS_THROUGH'} # return without finishing operator so that it can enter edit mode 

addon_keymaps = []

def register_keymap():
    wm = bpy.context.window_manager
    km = wm.keyconfigs.addon.keymaps.new(name='Object Mode', space_type='EMPTY')
    kmi = km.keymap_items.new(ApplyPrimitiveGeometryNodesModifier.bl_idname, 'TAB', 'PRESS')
    addon_keymaps.append((km, kmi))

def unregister_keymap():
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

def register():
    register_keymap()
    bpy.utils.register_class(AddNoneDestructivePrimitive)
    bpy.utils.register_class(ApplyPrimitiveGeometryNodesModifier)
    bpy.types.VIEW3D_MT_mesh_add.prepend(draw_add_none_destructive_primitive)

def unregister():
    unregister_keymap()
    bpy.utils.unregister_class(AddNoneDestructivePrimitive)
    bpy.utils.unregister_class(ApplyPrimitiveGeometryNodesModifier)
    bpy.types.VIEW3D_MT_mesh_add.remove(draw_add_none_destructive_primitive)